MODULE isfcav
   !!======================================================================
   !!                       ***  MODULE  isfcav  ***
   !! Ice shelf cavity module :  update ice shelf melting under ice
   !!                            shelf
   !!======================================================================
   !! History :  3.2  !  2011-02  (C.Harris  ) Original code isf cav
   !!            3.4  !  2013-03  (P. Mathiot) Merging + parametrization
   !!            4.1  !  2019-09  (P. Mathiot) Split ice shelf cavity and ice shelf parametrisation
   !!----------------------------------------------------------------------

   !!----------------------------------------------------------------------
   !!   isf_cav       : update ice shelf melting under ice shelf
   !!----------------------------------------------------------------------
   USE isf_oce        ! ice shelf public variables
   !
   USE isfrst   , ONLY: isfrst_read    ! ice shelf restart read/write subroutine
   USE isfutils , ONLY: debug          ! ice shelf debug subroutine
   USE isftbl   , ONLY: isf_tbl        ! ice shelf top boundary layer properties subroutine
   USE isfcavmlt, ONLY: isfcav_mlt     ! ice shelf melt formulation subroutine
   USE isfcavgam, ONLY: isfcav_gammats ! ice shelf melt exchange coeficient subroutine
   USE isfdiags , ONLY: isf_diags_flx  ! ice shelf diags subroutine
   USE zdfdrg   , ONLY: rCd0_top, r_ke0_top ! vertical physics: top/bottom drag coef.
   !
   USE oce      , ONLY: ts, uu, vv, rn2                 ! ocean dynamics and tracers
   USE dom_oce                                          ! ocean space and time domain
   USE par_oce                                          ! ocean space and time domain
   USE phycst   , ONLY: grav,rho0,rho0_rcp,r1_rho0_rcp  ! physical constants
   USE eosbn2   , ONLY: ln_teos10                       ! use ln_teos10 or not
   !
   USE in_out_manager ! I/O manager
   USE iom            ! I/O library
   USE fldread        ! read input field at current time step
   USE lbclnk         ! lbclnk
   USE lib_mpp        ! MPP library

   IMPLICIT NONE

   PRIVATE

   PUBLIC   isf_cav, isf_cav_init ! routine called in isfmlt

   !! * Substitutions   
#  include "do_loop_substitute.h90"
#  include "domzgr_substitute.h90"
   !!----------------------------------------------------------------------
   !! NEMO/OCE 4.0 , NEMO Consortium (2018)
   !! $Id: sbcisf.F90 10536 2019-01-16 19:21:09Z mathiot $
   !! Software governed by the CeCILL license (see ./LICENSE)
   !!----------------------------------------------------------------------
CONTAINS

   SUBROUTINE isf_cav( kt, Kmm, ptsc, pfwf )
      !!---------------------------------------------------------------------
      !!                     ***  ROUTINE isf_cav  ***
      !!
      !! ** Purpose :   handle surface boundary condition under ice shelf
      !!
      !! ** Method  :   based on Mathiot et al. (2017)
      !!
      !! ** Action  :   - compute geometry of the Losch top bournary layer (see Losch et al. 2008)
      !!                - depending on the chooses option
      !!                   - compute temperature/salt in the tbl
      !!                   - compute exchange coeficient
      !!                   - compute heat and fwf fluxes
      !!                   - output
      !! 
      !! ** Convention : all fluxes are from isf to oce
      !!
      !!---------------------------------------------------------------------
      INTEGER, INTENT(in) ::   kt   ! ocean time step
      INTEGER, INTENT(in) ::   Kmm  ! ocean time level index
      REAL(wp), DIMENSION(A2D(0),jpts), INTENT(inout) ::   ptsc   ! T & S ice shelf cavity contents
      REAL(wp), DIMENSION(A2D(0))     , INTENT(inout) ::   pfwf   ! ice shelf fwf
      !!---------------------------------------------------------------------
      LOGICAL  ::   lit
      INTEGER  ::   nit, ji, jj, ikt
      REAL(wp) ::   zerr
      REAL(wp) ::   zcoef, zdku, zdkv
      REAL(wp), DIMENSION(A2D(0))  ::   zqlat, zqoce, zqhc, zqh  ! heat fluxes
      REAL(wp), DIMENSION(A2D(0))  ::   zqh_b, zRc               !
      REAL(wp), DIMENSION(A2D(0))  ::   zgammat, zgammas         ! exchange coeficient
      REAL(wp), DIMENSION(A2D(0))  ::   zttbl, zstbl             ! temp. and sal. in top boundary layer
      REAL(wp), DIMENSION(A2D(0))  ::   zustar                   ! u*
      REAL(wp), DIMENSION(jpi,jpj) ::   zutbl, zvtbl             ! top boundary layer velocity
      !!---------------------------------------------------------------------
      !
      !===============================
      ! 1.: compute T and S in the tbl
      !===============================
      CALL isf_tbl( Kmm, ts(A2D(0),:,jp_tem,Kmm), 'T', misfkt_cav, rhisf_tbl_cav, & ! <<== in
         &                                                                 zttbl, & ! ==>> out
         &                                             misfkb_cav, rfrac_tbl_cav  ) ! <<== in (optional)
      !
      CALL isf_tbl( Kmm, ts(A2D(0),:,jp_sal,Kmm), 'T', misfkt_cav, rhisf_tbl_cav, & ! <<== in
         &                                                                 zstbl, & ! ==>> out
         &                                              misfkb_cav, rfrac_tbl_cav ) ! <<== in (optional)
      !
      ! output T/S for the top boundary layer
      CALL iom_put( 'ttbl_cav', zttbl(:,:) * mskisf_cav(:,:) )
      CALL iom_put( 'stbl'    , zstbl(:,:) * mskisf_cav(:,:) )
      !
      !==========================================
      ! 2.: compute velocity in the tbl if needed
      !==========================================
      IF ( TRIM(cn_gammablk) == 'vel_stab' .OR. TRIM(cn_gammablk) == 'vel' ) THEN
         ! compute velocity in tbl
         CALL isf_tbl( Kmm, uu(A2D(0),:,Kmm), 'U', miku(A2D(0)), rhisf_tbl_cav, & ! <<== in
            &                                                     zutbl(A2D(0)) ) ! ==>> out
         !
         CALL isf_tbl( Kmm, vv(A2D(0),:,Kmm), 'V', mikv(A2D(0)), rhisf_tbl_cav, & ! <<== in
            &                                                     zvtbl(A2D(0)) ) ! ==>> out
         !
         ! compute ustar (AD15 eq. 27)
         CALL lbc_lnk( 'isfcav', zutbl, 'U', -1._wp, zvtbl, 'V', -1._wp )
         DO_2D( 0, 0, 0, 0 )         
            zustar(ji,jj) = SQRT( rCd0_top(ji,jj) * (  0.25_wp * (  (zutbl(ji,jj)+zutbl(ji-1,jj))*(zutbl(ji,jj)+zutbl(ji-1,jj))   &
               &                                                  + (zvtbl(ji,jj)+zvtbl(ji,jj-1))*(zvtbl(ji,jj)+zvtbl(ji,jj-1)) ) &
               &                                     + r_ke0_top ) ) * mskisf_cav(ji,jj)
         END_2D
         !
         ! output
         CALL iom_put( 'isfustar', zustar(:,:)    )
         CALL iom_put( 'utbl'    , zutbl(A2D(0))  )
         CALL iom_put( 'vtbl'    , zvtbl(A2D(0))  )
         !
      ELSE
         zustar(:,:) = 0._wp ! not used
      ENDIF
      !
      !==============================
      ! 3.: compute ice shelf melting
      !==============================
      ! --- initialisation --- !
      IF ( TRIM(cn_gammablk) == 'vel_stab' ) THEN
         !
         DO_2D( 0, 0, 0, 0 )
            zqoce(ji,jj) = -pfwf(ji,jj) * rLfusisf
            zqh_b(ji,jj) =  ptsc(ji,jj,jp_tem) * rho0_rcp ! last time step total heat fluxes (to speed up convergence)

            ikt = mikt(ji,jj)
            ! compute Rc number (as done in zdfric.F90)
!!gm better to do it like in the new zdfric.F90   i.e. avm weighted Ri computation
            zcoef = 0.5_wp / e3w(ji,jj,ikt+1,Kmm)
            !                                            ! shear of horizontal velocity
            zdku = zcoef * (  ( uu(ji-1,jj  ,ikt  ,Kmm) + uu(ji,jj,ikt  ,Kmm) )  &   ! add () for NP repro
               &            - ( uu(ji-1,jj  ,ikt+1,Kmm) + uu(ji,jj,ikt+1,Kmm) )  )
            zdkv = zcoef * (  ( vv(ji  ,jj-1,ikt  ,Kmm) + vv(ji,jj,ikt  ,Kmm) )  &   ! add () for NP repro
               &            - ( vv(ji  ,jj-1,ikt+1,Kmm) + vv(ji,jj,ikt+1,Kmm) )  )
            !                                            ! richardson number (minimum value set to zero)
            zRc(ji,jj) = MAX( rn2(ji,jj,ikt+1), 1.e-20_wp ) / MAX( zdku*zdku + zdkv*zdkv, 1.e-20_wp )
         END_2D
         !
      ENDIF
      !
      ! --- iteration loop --- !
      nit = 1 ; lit = .TRUE.
      DO WHILE ( lit )    ! maybe just a constant number of iteration as in blk_core is fine
         !
         ! compute gammat everywhere (2d)
         ! useless if melt specified
         IF ( TRIM(cn_isfcav_mlt) .NE. 'spe' ) THEN
            CALL isfcav_gammats( Kmm, zustar, zttbl, zstbl, zqoce, pfwf, zRc,  & ! <<== in
               &                                             zgammat, zgammas  ) ! ==>> out
         END IF
         !
         ! compute tfrz, latent heat and melt (2d)
         CALL isfcav_mlt(kt, zgammat, zgammas, zttbl, zstbl, & ! <<== in
            &                         zqhc   , zqoce, pfwf   ) ! ==>> out
         !
         ! define if we need to iterate
         SELECT CASE ( cn_gammablk )
         CASE ( 'spe', 'vel' )
            ! no convergence needed
            lit = .FALSE.
         CASE ( 'vel_stab' )
            ! compute error between 2 iterations
            zerr = 0._wp
            DO_2D( 0, 0, 0, 0 )
               zerr = MAX( zerr, ABS( zqhc(ji,jj) + zqoce(ji,jj) - zqh_b(ji,jj) ) )
            END_2D
            CALL mpp_max( 'isfcav', zerr )   ! max over the global domain
            !
            ! define if iteration needed
            IF( nit >= 100 ) THEN              ! too much iteration
               CALL ctl_stop( 'STOP', 'isf_cav: vel_stab gamma formulation had too many iterations ...' )
            ELSEIF( zerr <= 0.01_wp ) THEN     ! convergence is achieve
               lit = .FALSE.
            ELSE                               ! converge is not yet achieve
               nit = nit + 1
               zqh_b(:,:) = zqhc(:,:) + zqoce(:,:)
            END IF
         END SELECT
         !
      END DO
      !
      DO_2D( 0, 0, 0, 0 )
         ! compute heat and water flux ( > 0 from isf to oce)
         pfwf (ji,jj) = pfwf (ji,jj) * mskisf_cav(ji,jj)
         zqoce(ji,jj) = zqoce(ji,jj) * mskisf_cav(ji,jj)
         zqhc (ji,jj) = zqhc (ji,jj) * mskisf_cav(ji,jj)
         !
         ! compute heat content flux ( > 0 from isf to oce)
         zqlat(ji,jj) = - pfwf(ji,jj) * rLfusisf    ! 2d latent heat flux (W/m2)
         !
         ! total heat flux ( > 0 from isf to oce)
         zqh(ji,jj) = ( zqhc(ji,jj) + zqoce(ji,jj) )
         !
         ! set temperature content
         ptsc(ji,jj,jp_tem) = zqh(ji,jj) * r1_rho0_rcp
      END_2D
      !
      ! output fluxes
      CALL isf_diags_flx( Kmm, misfkt_cav, misfkb_cav, rhisf_tbl_cav, rfrac_tbl_cav, 'cav', pfwf, zqoce, zqlat, zqhc )
      !
      ! --- output and debug --- !
      !
      CALL iom_put( 'isfgammat', zgammat(:,:) )
      CALL iom_put( 'isfgammas', zgammas(:,:) )
      !
      IF ( ln_isfdebug ) THEN
         IF(lwp) WRITE(numout,*) ''
         CALL debug( 'isfcav gammaT:', zgammat(:,:) )
         CALL debug( 'isfcav gammaS:', zgammas(:,:) )
         CALL debug('isf_cav: ptsc T',ptsc(:,:,1))
         CALL debug('isf_cav: ptsc S',ptsc(:,:,2))
         CALL debug('isf_cav: pfwf fwf',pfwf(:,:))
         IF(lwp) WRITE(numout,*) ''
      END IF
      !
   END SUBROUTINE isf_cav

   SUBROUTINE isf_cav_init
      !!---------------------------------------------------------------------
      !!                  ***  ROUTINE isf_cav_init ***
      !!
      !! ** Purpose : initialisation of variable needed to compute melt under an ice shelf
      !!
      !!----------------------------------------------------------------------
      INTEGER ::   ierr
      INTEGER ::   ji, jj     ! dummy loop indices
      !!---------------------------------------------------------------------
      !
      !==============
      ! 0: allocation
      !==============
      CALL isf_alloc_cav()
      !
      !==================
      ! 1: initialisation
      !==================
      DO_2D( 0, 0, 0, 0 )
         ! top and bottom level of the 'top boundary layer'
         misfkt_cav(ji,jj)    = mikt(ji,jj)
         misfkb_cav(ji,jj)    = 1
         !
         ! thickness of 'tbl' and fraction of bottom cell affected by 'tbl'
         rhisf_tbl_cav(ji,jj) = 0.0_wp
         rfrac_tbl_cav(ji,jj) = 0.0_wp
         !
         ! cavity mask
         mskisf_cav(ji,jj) = ( 1._wp - tmask(ji,jj,1) ) * ssmask(ji,jj)
         !
      END_2D
      !
#if ! defined key_RK3
      !================
      ! 2: activate restart
      !================
      !
      !================
      ! 3: read restart
      !================
      !
      ! MLF: read cav variable from restart
      IF ( ln_rstart ) CALL isfrst_read('cav', risf_cav_tsc, fwfisf_cav, risf_cav_tsc_b, fwfisf_cav_b)
      !
#endif
      !==========================================
      ! 3: specific allocation and initialisation (depending of scheme choice)
      !==========================================
      !
      SELECT CASE ( TRIM(cn_isfcav_mlt) )
      CASE( 'spe' )

         ALLOCATE( sf_isfcav_fwf(1), STAT=ierr )
         ALLOCATE( sf_isfcav_fwf(1)%fnow(A2D(0),1), sf_isfcav_fwf(1)%fdta(A2D(0),1,2) )
         CALL fld_fill( sf_isfcav_fwf, (/ sn_isfcav_fwf /), cn_isfdir, 'isf_cav_init', 'read fresh water flux isf data', 'namisf' )

         IF(lwp) WRITE(numout,*)
         IF(lwp) WRITE(numout,*) '  ==>> The ice shelf melt inside the cavity is read from forcing files'

      CASE( '2eq' )
         IF(lwp) WRITE(numout,*)
         IF(lwp) WRITE(numout,*) '  ==>> The original ISOMIP melt formulation is used to compute melt under the ice shelves'

      CASE( '3eq' )
         ! coeficient for linearisation of potential tfreez
         ! Crude approximation for pressure (but commonly used)
         IF ( ln_teos10 ) THEN   ! linearisation from Jourdain et al. (2017)
            risf_lamb1 =-0.0564_wp
            risf_lamb2 = 0.0773_wp
            risf_lamb3 =-7.8633e-8 * grav * rho0
         ELSE                  ! linearisation from table 4 (Asay-Davis et al., 2015)
            risf_lamb1 =-0.0573_wp
            risf_lamb2 = 0.0832_wp
            risf_lamb3 =-7.5300e-8 * grav * rho0
         ENDIF

         IF(lwp) WRITE(numout,*)
         IF(lwp) WRITE(numout,*) '  ==>> The 3 equations melt formulation is used to compute melt under the ice shelves'

      CASE DEFAULT
         CALL ctl_stop(' cn_isfcav_mlt method unknown (spe, 2eq, 3eq), check namelist')
      END SELECT
      !
   END SUBROUTINE isf_cav_init

END MODULE isfcav
